==> kink-0.0.1-alpha-sources.jar: org/kink_lang/kink/hostfun/graph/impl/ArgsCollector.java <==
package org.kink_lang.kink.hostfun.graph.impl;

import java.util.ArrayList;
import java.util.List;

import org.kink_lang.kink.hostfun.HostContext;
import org.kink_lang.kink.FunVal;
import org.kink_lang.kink.hostfun.HostResult;
import org.kink_lang.kink.Val;
import org.kink_lang.kink.Vm;
import org.kink_lang.kink.internal.function.ThrowingFunction2;
import org.kink_lang.kink.hostfun.graph.GraphNode;

/**
 * Helper class to collect args from the graph nodes.
 */
class ArgsCollector {

    /** The vm. */
    private final Vm vm;

    /** The graph nodes of the args. */
    private final GraphNode[] args;

    /**
     * Constructs a collector.
     */
    ArgsCollector(Vm vm, GraphNode[] args) {
        this.vm = vm;
        this.args = args;
    }

    /**
     * Collects args and emits to the handler.
     * @param c
     * @param handler
     */
    HostResult collectArgs(HostContext c,
                           ThrowingFunction2<HostContext, Val[], HostResult> handler) {
        return collectArgsLoop(c, List.of(), handler);
    }

    /**
     * Loop of collectArgs.
     */
    private HostResult collectArgsLoop(HostContext c,
                                       List<Val> collectedVals,
                                       ThrowingFunction2<HostContext, Val[], HostResult> handler) {
        if (collectedVals.size() == args.length) {
            Val[] argVals = collectedVals.toArray(new Val[0]);
            FunVal caller = vm.fun.make().action(cc -> handler.apply(cc, argVals));
            return c.call(caller);
        }

        GraphNode argGraph = args[collectedVals.size()];
        return c.call(argGraph).on((cc, argVal) -> {
            List<Val> newVals = new ArrayList<>(collectedVals.size() + 1);
            newVals.addAll(collectedVals);
            newVals.add(argVal);
            return collectArgsLoop(cc, newVals, handler);
        });
    }

}

// vim: et sw=4 sts=4 fdm=marker

==> kink-0.8.3-alpha-sources.jar: org/kink_lang/kink/hostfun/graph/impl/ArgsCollector.java <==
package org.kink_lang.kink.hostfun.graph.impl;

import java.util.ArrayList;
import java.util.List;

import org.kink_lang.kink.hostfun.HostContext;
import org.kink_lang.kink.hostfun.HostResult;
import org.kink_lang.kink.Val;
import org.kink_lang.kink.internal.function.ThrowingFunction2;
import org.kink_lang.kink.hostfun.graph.GraphNode;

/**
 * Helper class to collect args from the graph nodes.
 */
class ArgsCollector {

    /** The graph nodes of the args. */
    private final GraphNode[] args;

    /**
     * Constructs a collector.
     */
    ArgsCollector(GraphNode[] args) {
        this.args = args;
    }

    /**
     * Collects args and emits to the handler.
     * @param c
     * @param handler
     */
    HostResult collectArgs(
            HostContext c,
            ThrowingFunction2<HostContext, Val[], HostResult> handler
            ) throws Throwable {
        return collectArgsLoop(c, List.of(), handler);
    }

    /**
     * Loop of collectArgs.
     */
    private HostResult collectArgsLoop(
            HostContext c,
            List<Val> collectedVals,
            ThrowingFunction2<HostContext, Val[], HostResult> handler
            ) throws Throwable {
        if (collectedVals.size() == args.length) {
            Val[] argVals = collectedVals.toArray(new Val[0]);
            return handler.apply(c, argVals);
        }

        GraphNode argGraph = args[collectedVals.size()];
        return c.call(argGraph).on((cc, argVal) -> {
            List<Val> newVals = new ArrayList<>(collectedVals.size() + 1);
            newVals.addAll(collectedVals);
            newVals.add(argVal);
            return collectArgsLoop(cc, newVals, handler);
        });
    }

}

// vim: et sw=4 sts=4 fdm=marker
